import platform


class GamepadReader:
    def __init__(self):
        self.__operating_system = platform.system()
        if self.__operating_system == "Windows":
            from mistercar_input_devices.backend.windows.adapters.input_readers.gamepad_reader_adapter import \
                GamepadReaderAdapter
            self.__reader = GamepadReaderAdapter()
        elif self.__operating_system == "Linux":
            from mistercar_input_devices.backend.linux.adapters.input_readers.gamepad_reader_adapter import \
                GamepadReaderAdapter
            self.__reader = GamepadReaderAdapter()
        elif self.__operating_system == "Darwin":
            from mistercar_input_devices.backend.macos.adapters.input_readers.gamepad_reader_adapter import \
                GamepadReaderAdapter
            self.__reader = GamepadReaderAdapter()

    # === STICKS ===
    def get_left_stick(self):
        """Returns (x, y) tuple for left stick"""
        return self.__reader.get_left_stick()

    def get_right_stick(self):
        """Returns (x, y) tuple for right stick"""
        return self.__reader.get_right_stick()

    def get_left_stick_x(self):
        return self.__reader.get_left_stick_x()

    def get_left_stick_y(self):
        return self.__reader.get_left_stick_y()

    def get_right_stick_x(self):
        return self.__reader.get_right_stick_x()

    def get_right_stick_y(self):
        return self.__reader.get_right_stick_y()

    # === TRIGGERS ===
    def get_triggers(self):
        """Returns (left, right) tuple"""
        return self.__reader.get_triggers()

    def get_left_trigger(self):
        return self.__reader.get_left_trigger()

    def get_right_trigger(self):
        return self.__reader.get_right_trigger()

    # === FACE BUTTONS ===
    def get_face_buttons(self):
        """Returns [A, B, X, Y] button states"""
        return self.__reader.get_face_buttons()

    def get_button_a(self):
        return self.__reader.get_button_a()

    def get_button_b(self):
        return self.__reader.get_button_b()

    def get_button_x(self):
        return self.__reader.get_button_x()

    def get_button_y(self):
        return self.__reader.get_button_y()

    # === SHOULDER BUTTONS ===
    def get_shoulder_buttons(self):
        """Returns [left, right] shoulder button states"""
        return self.__reader.get_shoulder_buttons()

    def get_left_shoulder(self):
        return self.__reader.get_left_shoulder()

    def get_right_shoulder(self):
        return self.__reader.get_right_shoulder()

    # === THUMB BUTTONS ===
    def get_thumb_buttons(self):
        """Returns [left, right] thumb button states"""
        return self.__reader.get_thumb_buttons()

    def get_left_thumb(self):
        return self.__reader.get_left_thumb()

    def get_right_thumb(self):
        return self.__reader.get_right_thumb()

    # === MENU BUTTONS ===
    def get_menu_buttons(self):
        """Returns [back, start] menu button states"""
        return self.__reader.get_menu_buttons()

    def get_back_button(self):
        return self.__reader.get_back_button()

    def get_start_button(self):
        return self.__reader.get_start_button()

    # === DPAD ===
    def get_dpad(self):
        """Returns [up, down, left, right] dpad states"""
        return self.__reader.get_dpad()

    def get_dpad_up(self):
        return self.__reader.get_dpad_up()

    def get_dpad_down(self):
        return self.__reader.get_dpad_down()

    def get_dpad_left(self):
        return self.__reader.get_dpad_left()

    def get_dpad_right(self):
        return self.__reader.get_dpad_right()

    # === KEYBOARD-STYLE METHODS ===
    def get_button_state(self, button_name):
        """Get state of a single button by name"""
        return self.__reader.get_button_state(button_name)

    def get_states_of_multiple_buttons(self, buttons_to_check):
        """Returns list of button states in same order as input"""
        return self.__reader.get_states_of_multiple_buttons(buttons_to_check)



import win32api as wapi
from mistercar_input_devices.backend.windows.platform_specific.keyboard.virtual_keys import vk


class KeyboardReader:
    """Read keyboard input"""

    def __init__(self):
        pass

    def get_key_state(self, key):
        if wapi.GetAsyncKeyState(vk[key]):
            return True
        else:
            return False

    def get_states_of_multiple_keys(self, keys_to_check):
        """Returns a vector of 0s and 1s describing which keys are pressed and which are not."""
        states = [0] * len(keys_to_check)
        for i in range(len(keys_to_check)):
            if wapi.GetAsyncKeyState(vk[keys_to_check[i]]):
                states[i] = 1
        return states



import platform


class GamepadEmulator:
    def __init__(self):
        self.__operating_system = platform.system()
        if self.__operating_system == "Windows":
            from mistercar_input_devices.backend.windows.adapters.input_emulators.gamepad_emulator_adapter import \
                GamepadEmulatorAdapter
            self.__emulator = GamepadEmulatorAdapter()
        elif self.__operating_system == "Linux":
            from mistercar_input_devices.backend.linux.adapters.input_emulators.gamepad_emulator_adapter import \
                GamepadEmulatorAdapter
            self.__emulator = GamepadEmulatorAdapter()
        elif self.__operating_system == "Darwin":
            from mistercar_input_devices.backend.macos.adapters.input_emulators.gamepad_emulator_adapter import \
                GamepadEmulatorAdapter
            self.__emulator = GamepadEmulatorAdapter()

    # === STICKS ===
    def emulate_left_stick(self, x, y):
        """Set left stick position"""
        return self.__emulator.emulate_left_stick(x, y)

    def emulate_right_stick(self, x, y):
        """Set right stick position"""
        return self.__emulator.emulate_right_stick(x, y)

    def emulate_left_stick_x(self, value):
        return self.__emulator.emulate_left_stick_x(value)

    def emulate_left_stick_y(self, value):
        return self.__emulator.emulate_left_stick_y(value)

    def emulate_right_stick_x(self, value):
        return self.__emulator.emulate_right_stick_x(value)

    def emulate_right_stick_y(self, value):
        return self.__emulator.emulate_right_stick_y(value)

    # === TRIGGERS ===
    def emulate_triggers(self, left, right):
        """Set both trigger values"""
        return self.__emulator.emulate_triggers(left, right)

    def emulate_left_trigger(self, value):
        return self.__emulator.emulate_left_trigger(value)

    def emulate_right_trigger(self, value):
        return self.__emulator.emulate_right_trigger(value)

    # === FACE BUTTONS ===
    def emulate_face_buttons(self, a, b, x, y):
        """Set all face button states"""
        return self.__emulator.emulate_face_buttons(a, b, x, y)

    def emulate_button_a(self, pressed):
        return self.__emulator.emulate_button_a(pressed)

    def emulate_button_b(self, pressed):
        return self.__emulator.emulate_button_b(pressed)

    def emulate_button_x(self, pressed):
        return self.__emulator.emulate_button_x(pressed)

    def emulate_button_y(self, pressed):
        return self.__emulator.emulate_button_y(pressed)

    # === SHOULDER BUTTONS ===
    def emulate_shoulder_buttons(self, left, right):
        """Set both shoulder button states"""
        return self.__emulator.emulate_shoulder_buttons(left, right)

    def emulate_left_shoulder(self, pressed):
        return self.__emulator.emulate_left_shoulder(pressed)

    def emulate_right_shoulder(self, pressed):
        return self.__emulator.emulate_right_shoulder(pressed)

    # === THUMB BUTTONS ===
    def emulate_thumb_buttons(self, left, right):
        """Set both thumb button states"""
        return self.__emulator.emulate_thumb_buttons(left, right)

    def emulate_left_thumb(self, pressed):
        return self.__emulator.emulate_left_thumb(pressed)

    def emulate_right_thumb(self, pressed):
        return self.__emulator.emulate_right_thumb(pressed)

    # === MENU BUTTONS ===
    def emulate_menu_buttons(self, back, start):
        """Set both menu button states"""
        return self.__emulator.emulate_menu_buttons(back, start)

    def emulate_back_button(self, pressed):
        return self.__emulator.emulate_back_button(pressed)

    def emulate_start_button(self, pressed):
        return self.__emulator.emulate_start_button(pressed)

    # === DPAD ===
    def emulate_dpad(self, up, down, left, right):
        """Set dpad state"""
        return self.__emulator.emulate_dpad(up, down, left, right)

    def emulate_dpad_up(self, pressed):
        return self.__emulator.emulate_dpad_up(pressed)

    def emulate_dpad_down(self, pressed):
        return self.__emulator.emulate_dpad_down(pressed)

    def emulate_dpad_left(self, pressed):
        return self.__emulator.emulate_dpad_left(pressed)

    def emulate_dpad_right(self, pressed):
        return self.__emulator.emulate_dpad_right(pressed)





class MouseReaderAdapter:
    def __init__(self):
        pass

    def get_cursor_position(self):
        raise NotImplementedError

    def get_relative_movement(self):
        raise NotImplementedError

    def get_button_states(self):
        raise NotImplementedError

    def get_wheel(self):
        raise NotImplementedError

    def get_horizontal_wheel(self):
        raise NotImplementedError



"""Read the current state of Xbox Controllers"""
from ctypes import *

# Xinput DLL
try:
    _xinput = windll.xinput1_4
except OSError as err:
    _xinput = windll.xinput1_3


class _xinput_gamepad(Structure):
    """CType XInput Gamepad Object"""
    _fields_ = [("wButtons", c_ushort), ("left_trigger", c_ubyte),
                ("right_trigger", c_ubyte), ("thumb_lx", c_short),
                ("thumb_ly", c_short), ("thumb_rx", c_short),
                ("thumb_ry", c_short)]

    fields = [f[0] for f in _fields_]

    def __dict__(self):
        return {field: self.__getattribute__(field) for field in self.fields}

    def __str__(self):
        return str(self.__dict__())

    def __getitem__(self, string):
        return self.__dict__()[string]


class _xinput_state(Structure):
    """CType XInput State Object"""
    _fields_ = [("dwPacketNumber", c_uint),
                ("XINPUT_GAMEPAD", _xinput_gamepad)]

    fields = fields = [f[0] for f in _fields_]

    def __dict__(self):
        return {field: self.__getattribute__(field) for field in self.fields}

    def __str__(self):
        return str(self.__dict__())

    def __getitem__(self, string):
        return self.__dict__()[string]


class rController(object):
    """XInput Controller State reading object"""

    # All possible button values
    _buttons = {
        'DPAD_UP': 0x0001,
        'DPAD_DOWN': 0x0002,
        'DPAD_LEFT': 0x0004,
        'DPAD_RIGHT': 0x0008,
        'START': 0x0010,
        'BACK': 0x0020,
        'LEFT_THUMB': 0x0040,
        'RIGHT_THUMB': 0x0080,
        'LEFT_SHOULDER': 0x0100,
        'RIGHT_SHOULDER': 0x0200,
        'A': 0x1000,
        'B': 0x2000,
        'X': 0x4000,
        'Y': 0x8000
    }

    def __init__(self, ControllerID):
        """Initialise Controller object
        ControllerID    Int     Position number of desired controller (order of connection)
        """
        self.ControllerID = ControllerID
        self.dwPacketNumber = c_uint()

    @property
    def gamepad(self):
        """Returns the current gamepad state. Buttons pressed is shown as a raw integer value.
        Use rController.buttons for a list of buttons pressed.
        """
        state = _xinput_state()
        _xinput.XInputGetState(self.ControllerID - 1, pointer(state))
        self.dwPacketNumber = state.dwPacketNumber

        return state.XINPUT_GAMEPAD

    @property
    def buttons(self):
        """Returns a list of buttons currently pressed"""
        return [name for name, value in rController._buttons.items()
                if self.gamepad.wButtons & value == value]


def main():
    """Test the functionality of the rController object"""
    import time

    print('Testing controller in position 1:')
    print('Running 3 x 3 seconds tests')

    # Initialise Controller
    con = rController(1)

    # Loop printing controller state and buttons held
    for i in range(3):
        print('Waiting...')
        time.sleep(1.5)
        print('State: ', con.gamepad)
        print('Buttons: ', con.buttons)
        time.sleep(0.5)

    print('Done!')


if __name__ == '__main__':
    main()



import platform
from typing import Dict, Tuple, Set
from mistercar_input_devices.backend.multiplatform.input_readers.wheel_reader.base import WheelDevice, WheelState, \
    WheelButton


class TMXWheel(WheelDevice):
    """Thrustmaster TMX Racing Wheel implementation"""

    def __init__(self):
        self._adapter = None

        if platform.system() == "Windows":
            from mistercar_input_devices.backend.windows.adapters.input_readers.wheel.thrustmaster.tmx_adapter import TMXWheelAdapter
            self._adapter = TMXWheelAdapter()
        elif platform.system() == "Linux":
            raise NotImplementedError("Linux support not implemented yet")
        elif platform.system() == "Darwin":
            raise NotImplementedError("macOS support not implemented yet")
        else:
            raise NotImplementedError("Unsupported platform")

    def set_pedal_mode(self, mode: str) -> None:
        """Set the pedal mode (normal or swapped).

        In normal mode:
            - Right pedal is throttle
            - Left pedal is clutch
        In swapped mode:
            - Right pedal is clutch
            - Left pedal is throttle

        Args:
            mode: Either "normal" or "swapped"

        Raises:
            ValueError: If mode is not "normal" or "swapped"
        """
        if self._adapter:
            self._adapter.set_pedal_mode(mode)

    def get_pedal_mode(self) -> str:
        """Get current pedal mode.

        Returns:
            str: Current pedal mode ("normal" or "swapped")
        """
        if self._adapter:
            return self._adapter.get_pedal_mode()
        return "normal"  # default mode

    def get_supported_buttons(self) -> Set[WheelButton]:
        """Get set of buttons supported by this wheel"""
        return {
            WheelButton.PADDLE_LEFT,  # Left gear paddle
            WheelButton.PADDLE_RIGHT,  # Right gear paddle
            WheelButton.BUTTON_1,  # Left inner (LB)
            WheelButton.BUTTON_2,  # Right inner (RB)
            WheelButton.BUTTON_3,  # Left outer
            WheelButton.BUTTON_4,  # Right outer
            WheelButton.VIEW,  # ⧉ button
            WheelButton.MENU,  # ≡ button
            WheelButton.XBOX_A,  # Green A
            WheelButton.XBOX_B,  # Red B
            WheelButton.XBOX_X,  # Blue X
            WheelButton.XBOX_Y,  # Yellow Y
            WheelButton.XBOX_GUIDE,  # Xbox/Power button
            WheelButton.DPAD_UP,
            WheelButton.DPAD_DOWN,
            WheelButton.DPAD_LEFT,
            WheelButton.DPAD_RIGHT,
        }

    def get_state(self) -> WheelState:
        """Get the current state of all wheel controls"""
        state = self._adapter.get_state()
        if state is None:
            return WheelState()
        return state

    def get_steering(self) -> float:
        """Get steering wheel position (-1.0 to 1.0)"""
        state = self.get_state()
        return state.steering

    def get_pedals(self) -> Tuple[float, float, float]:
        """Get pedal positions (throttle, brake, clutch) from 0.0 to 1.0"""
        state = self.get_state()
        return state.throttle, state.brake, state.clutch

    def get_buttons(self) -> Dict[WheelButton, bool]:
        """Get states of all buttons"""
        state = self.get_state()
        return state.buttons

    def __del__(self):
        """Cleanup when object is destroyed"""
        if self._adapter:
            self._adapter.close()



import platform


class KeyboardEmulator:
    def __init__(self):
        self.__operating_system = platform.system()
        if self.__operating_system == "Windows":
            from mistercar_input_devices.backend.windows.adapters.input_emulators.keyboard_emulator_adapter import KeyboardEmulatorAdapter
            self.__emulator = KeyboardEmulatorAdapter()
        elif self.__operating_system == "Linux":
            from mistercar_input_devices.backend.linux.adapters.input_emulators.keyboard_emulator_adapter import KeyboardEmulatorAdapter
            self.__emulator = KeyboardEmulatorAdapter()
        elif self.__operating_system == "Darwin":
            from mistercar_input_devices.backend.macos.adapters.input_emulators.keyboard_emulator_adapter import KeyboardEmulatorAdapter
            self.__emulator = KeyboardEmulatorAdapter()

    def emulate_key(self, key, value):
        self.__emulator.emulate_key(key, value)

    def emulate_multiple_keys(self, keys_list):
        self.__emulator.emulate_multiple_keys(keys_list)



from mistercar_input_devices.backend.windows.platform_specific.mouse import mouse_reader as wmr


class MouseReaderAdapter:
    def __init__(self):
        self.__mouse_reader = wmr.MouseReader()

    def get_cursor_position(self):
        return self.__mouse_reader.get_cursor_position()

    def get_relative_movement(self):
        return self.__mouse_reader.get_relative_movement()

    def get_button_states(self):
        return self.__mouse_reader.get_button_states()

    def get_wheel(self):
        return self.__mouse_reader.get_wheel()

    def get_horizontal_wheel(self):
        return self.__mouse_reader.get_horizontal_wheel()



from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Dict, Tuple, Set
from enum import IntEnum, auto


class WheelButton(IntEnum):
    """Standard wheel button mappings"""

    # Core buttons (present on most wheels)
    PADDLE_LEFT = auto()  # Left paddle shifter
    PADDLE_RIGHT = auto()  # Right paddle shifter
    BUTTON_1 = auto()  # Primary action button
    BUTTON_2 = auto()  # Secondary action button
    DPAD_UP = auto()
    DPAD_DOWN = auto()
    DPAD_LEFT = auto()
    DPAD_RIGHT = auto()

    # Extended buttons (common but not universal)
    BUTTON_3 = auto()
    BUTTON_4 = auto()
    VIEW = auto()  # ⧉ button (formerly SELECT/BACK)
    MENU = auto()  # ≡ button (formerly START)

    # Console-specific buttons (same physical positions, different labels)
    XBOX_A = auto()  # Xbox: Green A  | PlayStation: Blue Cross (×)
    XBOX_B = auto()  # Xbox: Red B    | PlayStation: Red Circle (○)
    XBOX_X = auto()  # Xbox: Blue X   | PlayStation: Pink Square (□)
    XBOX_Y = auto()  # Xbox: Yellow Y | PlayStation: Green Triangle (△)
    XBOX_GUIDE = auto()  # Xbox: Guide    | PlayStation: PS button

    # Advanced features
    CLUTCH_PADDLE_LEFT = auto()  # Additional clutch paddle
    CLUTCH_PADDLE_RIGHT = auto()  # Some high-end wheels have dual clutch
    DIAL_1 = auto()  # Rotary encoder/dial
    DIAL_2 = auto()  # Additional dial


@dataclass
class WheelState:
    """Represents the current state of a racing wheel"""
    # Main controls (-1.0 to 1.0 for steering, 0.0 to 1.0 for pedals)
    steering: float = 0.0  # -1.0 = full left, 1.0 = full right
    throttle: float = 0.0  # 0.0 = no throttle, 1.0 = full throttle
    brake: float = 0.0  # 0.0 = no brake, 1.0 = full brake
    clutch: float = 0.0  # 0.0 = no clutch, 1.0 = full clutch

    # Button states
    buttons: Dict[WheelButton, bool] = None

    def __post_init__(self):
        if self.buttons is None:
            self.buttons = {}

    def is_button_supported(self, button: WheelButton) -> bool:
        """Check if a button is supported by this wheel"""
        return button in self.buttons


class WheelDevice(ABC):
    """Abstract base class for all wheel devices"""

    @abstractmethod
    def get_supported_buttons(self) -> Set[WheelButton]:
        """Get set of buttons supported by this wheel"""
        pass

    @abstractmethod
    def get_state(self) -> WheelState:
        """Get the current state of all wheel controls"""
        pass

    @abstractmethod
    def get_steering(self) -> float:
        """Get steering wheel position (-1.0 to 1.0)"""
        pass

    @abstractmethod
    def get_pedals(self) -> Tuple[float, float, float]:
        """Get pedal positions (throttle, brake, clutch) from 0.0 to 1.0"""
        pass

    @abstractmethod
    def get_buttons(self) -> Dict[WheelButton, bool]:
        """Get states of all buttons"""
        pass

    def supports_force_feedback(self) -> bool:
        """Whether the device supports force feedback"""
        return False

    def set_force_feedback(self, strength: float) -> None:
        """Set force feedback strength if supported"""
        raise NotImplementedError("Force feedback not supported")



import platform
import sys


def get_analyzer():
    system = platform.system()

    if system == "Windows":
        from mistercar_input_devices.device_analysis.analyzers.windows.pywinusb_analyzer import PyWinUSBAnalyzer
        return PyWinUSBAnalyzer()
    elif system == "Linux":
        from mistercar_input_devices.device_analysis.analyzers.linux.evdev_analyzer import EvdevAnalyzer
        return EvdevAnalyzer()
    elif system == "Darwin":
        from mistercar_input_devices.device_analysis.analyzers.macos.iokit_analyzer import IOKitAnalyzer
        return IOKitAnalyzer()
    else:
        raise NotImplementedError(f"Platform {system} is not supported")


def print_data_changes(data: bytes, last_data: list):
    """Print bytes only when they change"""
    current_data = list(data)

    # Check if data changed
    if current_data != last_data:
        # Print bytes in hex with spacing every 4 bytes for readability
        hex_values = ""
        for i, b in enumerate(current_data):
            hex_values += f'{b:02X}'
            if i < len(current_data) - 1:  # Don't add space after last byte
                hex_values += " " if (i + 1) % 4 != 0 else "  "
        print(hex_values)

        # Update last_data for next comparison
        last_data.clear()
        last_data.extend(current_data)


def main():
    try:
        analyzer = get_analyzer()
    except NotImplementedError as e:
        print(f"Error: {e}")
        sys.exit(1)
    except ImportError as e:
        print(f"Error: Required dependencies not installed for this platform: {e}")
        sys.exit(1)

    # List available devices
    print("Available devices:")
    try:
        devices = analyzer.list_devices()
        for i, device in enumerate(devices):
            print(f"{i + 1}. {device}")
    except Exception as e:
        print(f"Error listing devices: {e}")
        return

    # Get device selection from user
    if not devices:
        print("No devices found!")
        return

    while True:
        try:
            selection = int(input("\nSelect device number (or 0 to exit): "))
            if selection == 0:
                return
            if 1 <= selection <= len(devices):
                break
        except ValueError:
            pass
        print("Invalid selection!")

    device = devices[selection - 1]

    # Open selected device
    print(f"\nOpening {device.name}...")
    try:
        if not analyzer.open_device(device.vendor_id, device.product_id):
            print("Failed to open device!")
            return
    except Exception as e:
        print(f"Error opening device: {e}")
        return

    print("Device opened successfully!")
    print("\nMonitoring data changes... (Press Ctrl+C to exit)\n")

    try:
        # Store last data for comparison
        last_data = []
        analyzer.start_monitoring(lambda data: print_data_changes(data, last_data))

        # Keep main thread alive until Ctrl+C
        while True:
            pass

    except KeyboardInterrupt:
        print("\nStopping...")
    except Exception as e:
        print(f"\nError during monitoring: {e}")
    finally:
        analyzer.close_device()


if __name__ == "__main__":
    if sys.platform != "win32":
        print("This example only works on Windows!")
        sys.exit(1)
    main()



from mistercar_input_devices.backend.windows.platform_specific.pyxinput import rController


class GamepadReaderAdapter:
    def __init__(self):
        self.__read_controller = rController(1)

    # === STICKS ===
    def get_left_stick(self):
        """Returns (x, y) tuple for left stick"""
        gamepad = self.__read_controller.gamepad
        return (gamepad["thumb_lx"], gamepad["thumb_ly"])

    def get_right_stick(self):
        """Returns (x, y) tuple for right stick"""
        gamepad = self.__read_controller.gamepad
        return (gamepad["thumb_rx"], gamepad["thumb_ry"])

    def get_left_stick_x(self):
        return self.__read_controller.gamepad["thumb_lx"]

    def get_left_stick_y(self):
        return self.__read_controller.gamepad["thumb_ly"]

    def get_right_stick_x(self):
        return self.__read_controller.gamepad["thumb_rx"]

    def get_right_stick_y(self):
        return self.__read_controller.gamepad["thumb_ry"]

    # === TRIGGERS ===
    def get_triggers(self):
        """Returns (left, right) tuple"""
        gamepad = self.__read_controller.gamepad
        return (gamepad["left_trigger"], gamepad["right_trigger"])

    def get_left_trigger(self):
        return self.__read_controller.gamepad["left_trigger"]

    def get_right_trigger(self):
        return self.__read_controller.gamepad["right_trigger"]

    # === FACE BUTTONS ===
    def get_face_buttons(self):
        """Returns [A, B, X, Y] button states"""
        buttons = self.__read_controller.buttons
        return [
            1 if 'A' in buttons else 0,
            1 if 'B' in buttons else 0,
            1 if 'X' in buttons else 0,
            1 if 'Y' in buttons else 0
        ]

    def get_button_a(self):
        return 1 if 'A' in self.__read_controller.buttons else 0

    def get_button_b(self):
        return 1 if 'B' in self.__read_controller.buttons else 0

    def get_button_x(self):
        return 1 if 'X' in self.__read_controller.buttons else 0

    def get_button_y(self):
        return 1 if 'Y' in self.__read_controller.buttons else 0

    # === SHOULDER BUTTONS ===
    def get_shoulder_buttons(self):
        """Returns [left, right] shoulder button states"""
        buttons = self.__read_controller.buttons
        return [
            1 if "LEFT_SHOULDER" in buttons else 0,
            1 if "RIGHT_SHOULDER" in buttons else 0
        ]

    def get_left_shoulder(self):
        return 1 if "LEFT_SHOULDER" in self.__read_controller.buttons else 0

    def get_right_shoulder(self):
        return 1 if "RIGHT_SHOULDER" in self.__read_controller.buttons else 0

    # === THUMB BUTTONS ===
    def get_thumb_buttons(self):
        """Returns [left, right] thumb button states"""
        buttons = self.__read_controller.buttons
        return [
            1 if "LEFT_THUMB" in buttons else 0,
            1 if "RIGHT_THUMB" in buttons else 0
        ]

    def get_left_thumb(self):
        return 1 if "LEFT_THUMB" in self.__read_controller.buttons else 0

    def get_right_thumb(self):
        return 1 if "RIGHT_THUMB" in self.__read_controller.buttons else 0

    # === MENU BUTTONS ===
    def get_menu_buttons(self):
        """Returns [back, start] menu button states"""
        buttons = self.__read_controller.buttons
        return [
            1 if "BACK" in buttons else 0,
            1 if "START" in buttons else 0
        ]

    def get_back_button(self):
        return 1 if "BACK" in self.__read_controller.buttons else 0

    def get_start_button(self):
        return 1 if "START" in self.__read_controller.buttons else 0

    # === DPAD ===
    def get_dpad(self):
        """Returns [up, down, left, right] dpad states"""
        buttons = self.__read_controller.buttons
        return [
            1 if "DPAD_UP" in buttons else 0,
            1 if "DPAD_DOWN" in buttons else 0,
            1 if "DPAD_LEFT" in buttons else 0,
            1 if "DPAD_RIGHT" in buttons else 0
        ]

    def get_dpad_up(self):
        return 1 if "DPAD_UP" in self.__read_controller.buttons else 0

    def get_dpad_down(self):
        return 1 if "DPAD_DOWN" in self.__read_controller.buttons else 0

    def get_dpad_left(self):
        return 1 if "DPAD_LEFT" in self.__read_controller.buttons else 0

    def get_dpad_right(self):
        return 1 if "DPAD_RIGHT" in self.__read_controller.buttons else 0

    # === KEYBOARD-STYLE METHODS ===
    def get_button_state(self, button_name):
        """Get state of a single button by name"""
        button_mapping = {
            'A': 'A', 'B': 'B', 'X': 'X', 'Y': 'Y',
            "LEFT_SHOULDER": "LEFT_SHOULDER", "RIGHT_SHOULDER": "RIGHT_SHOULDER",
            "LEFT_THUMB": "LEFT_THUMB", "RIGHT_THUMB": "RIGHT_THUMB",
            "BACK": "BACK", "START": "START",
            "DPAD_UP": "DPAD_UP", "DPAD_DOWN": "DPAD_DOWN",
            "DPAD_LEFT": "DPAD_LEFT", "DPAD_RIGHT": "DPAD_RIGHT"
        }

        if button_name in button_mapping:
            return 1 if button_mapping[button_name] in self.__read_controller.buttons else 0
        return 0

    def get_states_of_multiple_buttons(self, buttons_to_check):
        """Returns list of button states in same order as input"""
        return [self.get_button_state(button) for button in buttons_to_check]





class GamepadReaderAdapter:
    def __init__(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    # === STICKS ===
    def get_left_stick(self):
        """Returns (x, y) tuple for left stick"""
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_right_stick(self):
        """Returns (x, y) tuple for right stick"""
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_left_stick_x(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_left_stick_y(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_right_stick_x(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_right_stick_y(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    # === TRIGGERS ===
    def get_triggers(self):
        """Returns (left, right) tuple"""
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_left_trigger(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_right_trigger(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    # === FACE BUTTONS ===
    def get_face_buttons(self):
        """Returns [A, B, X, Y] button states"""
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_button_a(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_button_b(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_button_x(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_button_y(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    # === SHOULDER BUTTONS ===
    def get_shoulder_buttons(self):
        """Returns [left, right] shoulder button states"""
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_left_shoulder(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_right_shoulder(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    # === THUMB BUTTONS ===
    def get_thumb_buttons(self):
        """Returns [left, right] thumb button states"""
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_left_thumb(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_right_thumb(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    # === MENU BUTTONS ===
    def get_menu_buttons(self):
        """Returns [back, start] menu button states"""
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_back_button(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_start_button(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    # === DPAD ===
    def get_dpad(self):
        """Returns [up, down, left, right] dpad states"""
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_dpad_up(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_dpad_down(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_dpad_left(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_dpad_right(self):
        raise NotImplementedError("macOS gamepad support not yet implemented")

    # === KEYBOARD-STYLE METHODS ===
    def get_button_state(self, button_name):
        """Get state of a single button by name"""
        raise NotImplementedError("macOS gamepad support not yet implemented")

    def get_states_of_multiple_buttons(self, buttons_to_check):
        """Returns list of button states in same order as input"""
        raise NotImplementedError("macOS gamepad support not yet implemented")



import pywinusb.hid as hid
from typing import List, Optional
from mistercar_input_devices.device_analysis.analyzers.base import DeviceAnalyzer, DeviceInfo


class PyWinUSBAnalyzer(DeviceAnalyzer):
    """Device analyzer using PyWinUSB"""

    def __init__(self):
        super().__init__()
        self._device = None
        self._last_data = None

    def list_devices(self) -> List[DeviceInfo]:
        devices = []
        for device in hid.find_all_hid_devices():
            devices.append(DeviceInfo(
                vendor_id=device.vendor_id,
                product_id=device.product_id,
                name=device.product_name or "Unknown Device",
                description=f"Vendor: {device.vendor_name}",
                path=device.device_path
            ))
        return devices

    def open_device(self, vendor_id: int, product_id: int) -> bool:
        """Open a specific device for analysis"""
        # Close any existing device first
        self.close_device()

        # Find and open the device
        try:
            all_devices = hid.HidDeviceFilter(
                vendor_id=vendor_id,
                product_id=product_id
            ).get_devices()

            if not all_devices:
                return False

            self._device = all_devices[0]
            self._device.open()

            # Store device info
            self.device_info = DeviceInfo(
                vendor_id=self._device.vendor_id,
                product_id=self._device.product_id,
                name=self._device.product_name or "Unknown Device",
                description=f"Vendor: {self._device.vendor_name}",
                path=self._device.device_path
            )

            # Set up data handler
            self._device.set_raw_data_handler(self._data_handler)

            return True

        except Exception as e:
            print(f"Error opening device: {e}")
            self.close_device()
            return False

    def close_device(self) -> None:
        """Close the currently opened device"""
        if self._device:
            self._device.close()
            self._device = None
            self.device_info = None

    def read_data(self) -> Optional[bytes]:
        """Return the last received data packet"""
        return self._last_data

    def _data_handler(self, data):
        """Internal callback for PyWinUSB data"""
        # Convert data to bytes
        data_bytes = bytes(data)
        self._last_data = data_bytes

        # Call user callback if set
        if self.data_callback:
            self.data_callback(data_bytes)

    def __del__(self):
        """Cleanup when object is destroyed"""
        self.close_device()



import ctypes
from _ctypes import Structure, Union
from ctypes import wintypes


# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-mouseinput
class MOUSEINPUT(ctypes.Structure):
    _fields_ = (("dx", ctypes.c_long),
                ("dy", ctypes.c_long),
                ("mouseData", ctypes.c_ulong),
                ("dwFlags", ctypes.c_ulong),
                ("time", ctypes.c_ulong),
                ("dwExtraInfo", ctypes.POINTER(ctypes.c_ulong)))


# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-keybdinput
class KEYBDINPUT(ctypes.Structure):
    _fields_ = (("wVk", ctypes.c_ushort),
                ("wScan", ctypes.c_ushort),
                ("dwFlags", ctypes.c_ulong),
                ("time", ctypes.c_ulong),
                ("dwExtraInfo", ctypes.POINTER(ctypes.c_ulong)))


# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-hardwareinput
class HARDWAREINPUT(ctypes.Structure):
    _fields_ = (("uMsg", ctypes.c_ulong),
                ("wParamL", ctypes.c_ushort),
                ("wParamH", ctypes.c_ushort))


class _INPUTunion(ctypes.Union):
    _fields_ = (("mi", MOUSEINPUT),
                ("ki", KEYBDINPUT),
                ("hi", HARDWAREINPUT))


# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-input
class INPUT(ctypes.Structure):
    _fields_ = (("type", ctypes.c_ulong),
                ("union", _INPUTunion))


# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-rawinputdevice
class RAWINPUTDEVICE(Structure):
    _fields_ = (
        ("usUsagePage", wintypes.USHORT),
        ("usUsage", wintypes.USHORT),
        ("dwFlags", wintypes.DWORD),
        ("hwndTarget", wintypes.HWND)
    )


# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-rawinputheader
class RAWINPUTHEADER(Structure):
    _fields_ = (
        ("dwType", wintypes.DWORD),
        ("dwSize", wintypes.DWORD),
        ("hDevice", wintypes.HANDLE),
        ("wParam", wintypes.WPARAM)
    )


#
class RAWMOUSE_BUTTONS(Structure):
    _fields_ = (
        ("usButtonFlags", wintypes.USHORT),
        ("usButtonData", wintypes.USHORT)
    )


class RAWMOUSE_UNION(Union):
    _fields_ = (
        ("ulButtons", wintypes.ULONG),
        ("Buttons", RAWMOUSE_BUTTONS)
    )


# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-rawmouse
class RAWMOUSE(Structure):
    _fields_ = (
        ("usFlags", wintypes.USHORT),
        ("union", RAWMOUSE_UNION),
        ("ulRawButtons", wintypes.ULONG),
        ("lLastX", wintypes.LONG),
        ("lLastY", wintypes.LONG),
        ("ulExtraInformation", wintypes.ULONG)
    )


# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-rawkeyboard
class RAWKEYBOARD(Structure):
    _fields_ = (
        ("MakeCode", wintypes.USHORT),
        ("Flags", wintypes.USHORT),
        ("Reserved", wintypes.USHORT),
        ("VKey", wintypes.USHORT),
        ("Message", wintypes.UINT),
        ("ExtraInformation", wintypes.ULONG)
    )


# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-rawhid
class RAWHID(Structure):
    _fields_ = (
        ("dwSizeHid", wintypes.DWORD),
        ("dwCount", wintypes.DWORD),
        ("bRawData", wintypes.BYTE * 1)  # Array of bytes of size 1
    )


class RAWINPUT_DATA(Union):
    _fields_ = (
        ("mouse", RAWMOUSE),
        ("keyboard", RAWKEYBOARD),
        ("hid", RAWHID)
    )


# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-rawinput
class RAWINPUT(Structure):
    _fields_ = (
        ("header", RAWINPUTHEADER),
        ("data", RAWINPUT_DATA)
    )
